#
# drop-manager.bro
#
# Actually handles the drops and restores
#
# This is separated from drop.bro in order to handle drops being initiated by
# multiple sources, but it still works fine in a single bro config too.
#
#  Brian Tierney, LBNL
#

@load scan
@load drop

type drop_rec: record {
                drop_count: count &default=0 ;
                tot_drop_count: count &default=0 ;
                tot_restore_count: count &default=0 ;
        };
global drop_info: table[addr] of drop_rec &read_expire = 7 days  &persistent;


event DropEvent(a: addr, msg: string)
{
	#print fmt ("got drop event for addr %s", a);

	if (a !in drop_info) 
	{
		local rec: drop_rec;
		rec$drop_count=1;
		rec$tot_drop_count=0;
		rec$tot_restore_count=0;
		drop_info[a] = rec;

		#Debug
		#print fmt("AddressDropped: %s %s", a, msg);
         	system(fmt("%s %s", Drop::drop_connectivity_script, a));
         	NOTICE([$note=Drop::AddressDropped, $src=a,
         			     $msg=fmt("dropping address %s%s", a, msg)]);
	}
	else 
	{  
		# this will ensure its only dropped once in a cluster config
		if (drop_info[a]$drop_count == 0 )
		{
			++drop_info[a]$tot_drop_count;
			msg=fmt("%s, dropped %d times", msg, drop_info[a]$tot_drop_count);
			#Debug
       			#print fmt("RepeatAddressDropped: %s %s ", a, msg);
       			system(fmt("%s %s", Drop::drop_connectivity_script, a));
            		NOTICE([$note=Drop::RepeatAddressDropped, $src=a, $msg=msg]);
		} 
		++drop_info[a]$drop_count;
	} 
}

event RestoreEvent(a: addr, msg: string)
{
	#print fmt ("got restore event for addr %s", a);

	if (a in drop_info)
	{
		if (drop_info[a]$drop_count == 1)
		{
			++drop_info[a]$tot_restore_count;
			msg=fmt("%s, restored %d times", msg, drop_info[a]$tot_restore_count);
			#Debug
			#print fmt("AddressRestored: %s, %s", a, msg);
			system(fmt("%s %s", Drop::restore_connectivity_script, a));
            		NOTICE([$note=Drop::AddressRestored, $src=a, $msg=msg]);
        		    Scan::shut_down_thresh_reached[a] = F;
		}
		--drop_info[a]$drop_count;
	}
}
